#!/usr/bin/perl
use List::Util qw[min max];
$CNFHOME="/home/zywang/program/release/raptorx-ss3-ss8";
$PSIBLASTEXE="/home/zywang/program/ncbi-blast-2.2.28+/bin/psiblast";
$NR="/home/zywang/db/nr/nrfilt";

require "$CNFHOME/bin/BioDSSP.pm";

#check arguments
(@ARGV >= 1) or die <<"VEND";
Usage: run_raptorx-ss8.pl [sequence file]
sequence file should be a one-line file containing a peptide sequence.
VEND

#Deal with the filename of sequence.
@paths=split /\//,$ARGV[0];
$pathsnum=scalar(@paths)-1;
my @tmp=split /\./,$paths[$pathsnum]; #./
my $numtmp=scalar(@tmp)-2;
$numtmp = 0 if($numtmp<0);
$seqPrefix=join(".",@tmp[0..$numtmp]);


#Read sequence in FASTA format and check sequence
open FSEQ,"<$ARGV[0]" or die $!;
@seqlines=<FSEQ>;
$seq="";
for($i=0;$i<scalar(@seqlines);$i++)
{
    if($seqlines[$i]=~/^>/){}
    else{
	chomp($seqlines[$i]);
	$seqlines[$i]=~s/\s+//g;
	$seq=$seq.$seqlines[$i];
    }
}
$seq=uc $seq;
$seq=~m/[ARNDCQEGHILKMFPSTWYVX]*/g;
#$seq=~/[BDAC]*/g;
#print $seq," ","  $& ","\n";
die "The sequence contains unknown amino acids other than ARNDCQEGHILKMFPSTWYVX \n" if ($seq ne $&);
close FSEQ;

#make tmp sequence file and test working directory.
$jobid=int(rand(99999999));
$jobid="$seqPrefix.$jobid";
$FNTMPSEQ="cnfsseight.$jobid.seq";
$FNTINPUT="cnfsseight.$jobid.input";
open FTMPSEQ,">$FNTMPSEQ";
print FTMPSEQ "$seq\n";
close FTMPSEQ;

$FNTMPPSSM="cnfsseight.$jobid.seq.pssm";
#Run psiblast to generate PSSM
#print STDERR "$PSIBLASTEXE -d $NR -j 1 -h 0.001 -i $FNTMPSEQ -Q $FNTMPPSSM\n";
print STDERR "Running PSIBLAST...\n";
$tmp=`$PSIBLASTEXE -d $NR -j 5 -h 0.001 -i $FNTMPSEQ -Q $FNTMPPSSM`;
#$tmp=`$PSIBLASTEXE -db $NR -query  $FNTMPSEQ -num_iterations 5 -out_ascii_pssm  $FNTMPPSSM -inclusion_ethresh 0.001`;
#print STDERR $tmp;
#exit;
#Generate test input file
print STDERR "Generating features...\n";
$tmp=`$CNFHOME/bin/GenFeat_nodssp.pl $FNTMPSEQ $FNTMPPSSM $CNFHOME/data/p.p2.c.unit $FNTINPUT $CNFHOME`;

#print STDERR $tmp;
#Run CNF_predict
$FNTMPRST="cnfsseight.$jobid.result";
$FNTMPNULL="cnfsseight.$jobid.null";
open FTMPNULL,">$FNTMPNULL";
print FTMPNULL "0\n";
close FTMPNULL;
#exit;
print STDERR "Predicting using CNF model...\n";
$cmd="$CNFHOME/bin/bcnf_mpitp CONF $CNFHOME/data/3state.conf ACT null PREDICT1 1 FEATMASK 0 ALLRST all.rst.csv BIEN 0 TRAIN $FNTMPNULL TEST $FNTINPUT RESULT $FNTMPRST RESUME $CNFHOME/data/3state-model.p0-1300.1342925 &> cnfsseight.$jobid.stderr";
`$cmd`;

#`./bcnf_mpitp CONF CNF.ax.norm.conf ACT null PREDICT1 1 FEATMASK 0 ALLRST all.rst.csv BIEN 1 TRAIN train.null TEST output RESULT result.cnfss8  `;

#Parse the output of cnf ss eight prediction
open fhCNFOutput,"<$FNTMPRST" or die $!;
my $horiz;
my @hoConf=qw();



$rstline=<fhCNFOutput>;
$rstline=~s/^\s+//;
chomp($rstline);
@p=split/\s+/, $rstline;
my @alllab=split //, $p[6];
my @prob=@p[7..scalar(@p)-1];
close fhCNFOutput;

@alllab=convert_label_to_letter3(@alllab);
#Output the data in letter form, each line is 
# [number] [amino acid] [ss in eight letters]  [eight probability]
print STDERR "Formating results...\n";


open fhRESULT,">$seqPrefix.ss3";
@allseq=split //, $seq;
print fhRESULT "#RaptorX-SS3: three-class secondary structure prediction results\n";
print fhRESULT "#probabilities are in the order of H(alpha helix) E(beta strand) C(loops), the 3 secondary structure types used in PSIPRED\n\n";
for($i=0;$i<@allseq;$i++)
{
    
    print fhRESULT sprintf("%4d %s %s   ",$i+1,$allseq[$i], $alllab[$i]);
    for($k=0;$k<3;$k++)
    {
	print fhRESULT sprintf("%.3f ",$prob[$i*3+$k]);
    } 
    print fhRESULT "\n";
    push @hoConf,sprintf("%1d",int(10*max(@prob[$i*3..$i*3+2])-1));
} 
close fhRESULT;

open fhHO,">$seqPrefix.horiz";

print fhHO "#RaptorX-SS3: three-class secondary structure prediction results\n\n";

for(my $i=0;$i<@allseq;$i=$i+60)
{
    my $end;
    if($i+60>@allseq)
    {
	$end=@allseq-1;
    }
    else{
	$end=$i+60-1;
    }
    print fhHO "Conf: ",join("",@hoConf[$i..$end]),"\n";
    print fhHO "Pred: ",join("",@alllab[$i..$end]),"\n";
    print fhHO "  AA: ",join("",@allseq[$i..$end]),"\n";
    print fhHO "      ";
    for(my $j=$i+10;$j<=$end+1;$j=$j+10){
	print fhHO sprintf("%10d",$j);
    }
    print fhHO "\n"x3;
}
close fhHO;


print STDERR "DONE.\n";
`rm -rf output*`;
if(!defined $ARGV[1]){
`rm cnfsseight.$jobid.*`;
}

